import unittest
from ethiomultlib import ethiopian_multiplication, ethiopian_multiply_multiple # Import both functions if you keep multiple multiplication

class TestEthiopianMultiplication(unittest.TestCase):

    def test_valid_positive_integers(self):
        """Test with various valid positive integer inputs."""
        test_cases = [
            (2, 2, 4),
            (3, 3, 9),
            (5, 7, 35),
            (1, 100, 100),
            (100, 1, 100),
            (45, 12, 540),
            (17, 34, 578),
            (25, 40, 1000) # Multiple of 10
            (100, 100, 10000),
            (123, 456, 56088),
            (999, 999, 998001),
            (1234, 5678, 7006652),
            (111111111, 111111111, 12345678987654321) # Just way over 100,000,000 boundary
        ]
        for a, b, expected in test_cases:
            with self.subTest(f"Testing {a} * {b}"): # Subtests for better error messages
                self.assertEqual(ethiopian_multiplication(a, b), expected)

    def test_one_as_input(self):
        """Test cases where one of the inputs is 1."""
        self.assertEqual(ethiopian_multiplication(1, 100), 100, "1 * 100 should be 100")
        self.assertEqual(ethiopian_multiplication(100, 1), 100, "100 * 1 should be 100")
        self.assertEqual(ethiopian_multiplication(1, 1), 1, "1 * 1 should be 1")

    def test_edge_case_large_numbers(self):
        """Test with very large positive integers to check for potential overflow issues (if applicable in pure Python)."""
        large_num = 10**6  # 1 million
        self.assertEqual(ethiopian_multiplication(large_num, 2), large_num * 2, f"{large_num} * 2 should be correct")
        self.assertEqual(ethiopian_multiplication(large_num, large_num), large_num * large_num, f"{large_num} * {large_num} should be correct")

    def test_zero_and_negative_inputs(self):
        """Test error handling for zero and negative inputs (ValueError)."""
        invalid_inputs = [-1, 0, -10]
        for invalid_input in invalid_inputs:
            with self.subTest(f"Testing invalid input {invalid_input}"):
                with self.assertRaises(ValueError, msg=f"Should raise ValueError for input: {invalid_input}"):
                    ethiopian_multiplication(invalid_input, 10) # Test with invalid input as first arg
                with self.assertRaises(ValueError, msg=f"Should raise ValueError for input: {invalid_input}"):
                    ethiopian_multiplication(10, invalid_input) # Test with invalid input as second arg

    def test_non_integer_inputs(self):
        """Test error handling for non-integer inputs (TypeError)."""
        non_integer_inputs = [3.14, "hello", [1, 2], None]
        for non_integer_input in non_integer_inputs:
            with self.subTest(f"Testing non-integer input {type(non_integer_input).__name__}"):
                with self.assertRaises(TypeError, msg=f"Should raise TypeError for input type: {type(non_integer_input).__name__}"):
                    ethiopian_multiplication(non_integer_input, 5) # Test non-int as first arg
                with self.assertRaises(TypeError, msg=f"Should raise TypeError for input type: {type(non_integer_input).__name__}"):
                    ethiopian_multiplication(5, non_integer_input) # Test non-int as second arg


class TestEthiopianMultipleMultiplication(unittest.TestCase): # Separate test suite for multiple multiplication (if kept)

    def test_valid_multiple_numbers(self):
        test_cases_multiple = [
            ([2, 3, 4], 24),
            ([5, 5, 5], 125),
            ([1, 2, 3, 4, 5], 120),
            ([7, 8, 9], 504),
            ([10, 20, 30], 6000)
        ]
        for numbers, expected in test_cases_multiple:
            with self.subTest(f"Testing multiple numbers: {numbers}"):
                self.assertEqual(ethiopian_multiply_multiple(numbers), expected)

    def test_empty_list(self):
        """Test multiplication of an empty list (should return 1)."""
        self.assertEqual(ethiopian_multiply_multiple([]), 1, "Empty list product should be 1")

    def test_list_with_one_number(self):
        """Test multiplication of a list with a single number."""
        self.assertEqual(ethiopian_multiply_multiple([15]), 15, "Single number list should return the number itself")

    def test_multiple_error_handling(self): # Error handling for multiple multiplication
        invalid_lists = [
            [2, -3, 4],  # Negative number in list
            [1, 2, 0, 4], # Zero in list
            [1, 2, 3.14], # Float in list
            [1, "a", 3]  # String in list
        ]
        for invalid_list in invalid_lists:
            with self.subTest(f"Testing invalid list: {invalid_list}"):
                with self.assertRaises((ValueError, TypeError)): # Expecting either ValueError or TypeError
                    ethiopian_multiply_multiple(invalid_list)


if __name__ == '__main__':
    unittest.main()